#!/usr/bin/env Rscript
library(tidyverse)

args <- commandArgs(trailingOnly = TRUE)

inputfile  <- args[[1]]
outputfile <- args[[2]]

config <- yaml::read_yaml("config.yml")

# read and process input data
df <- readRDS(inputfile) %>%
  mutate(
    Outcomes.DerivedCompositeGOSE = factor(
      Outcomes.DerivedCompositeGOSE,
      levels  = c(1, 3:8), # 2 is too infrequent to be fitted
      ordered = TRUE
    )
  )

impute_locf <- function(t, GOSE, tolerance, t_pred) {
  res  <- numeric(length(t_pred))
  res[] <- NA_integer_
  for (i in 1:length(t_pred)) {
    res[i] <- GOSE[t < t_pred[i] + tolerance] %>%
      tail(1) %>% # use last if multiple available
      ifelse(length(.) == 0, NA, .)
  }
  return(res %>% as.integer)
}

df_posteriors <- df %>%
  group_by(gupi) %>%
  arrange(gupi, Outcomes.DerivedCompositeGOSE) %>%
  do(
    t = config$t_out,
    GOSE_predicted = impute_locf(.$Outcomes.DerivedCompositeGOSEDaysPostInjury, .$Outcomes.DerivedCompositeGOSE %>% as.character %>% as.integer, config$t_buffer, config$t_out)
  ) %>%
  unnest %>%
  mutate(
    `1` = (GOSE_predicted == 1) %>% as.numeric,
    `2` = (GOSE_predicted == 2) %>% as.numeric,
    `3` = (GOSE_predicted == 3) %>% as.numeric,
    `4` = (GOSE_predicted == 4) %>% as.numeric,
    `5` = (GOSE_predicted == 5) %>% as.numeric,
    `6` = (GOSE_predicted == 6) %>% as.numeric,
    `7` = (GOSE_predicted == 7) %>% as.numeric,
    `8` = (GOSE_predicted == 8) %>% as.numeric
  ) %>%
  select(-GOSE_predicted) %>%
  gather(GOSE, probability, `1`:`8`) %>%
  mutate(
    gupi = as.character(gupi),
    GOSE = as.integer(GOSE)
  ) %>%
  arrange(gupi, t, GOSE)

saveRDS(df_posteriors, outputfile)
